@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001-2012, 2014-2018 Free Software Foundation,
-   Inc.
+   Copyright (C) 1999-2020 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -501,7 +500,7 @@ static struct mu_cfg_param imap4d_srv_param[] = {
     NULL, mu_offsetof (struct imap4d_srv_config, tls_mode), cb_tls,
     N_("Kind of TLS encryption to use for this server"),
     /* TRANSLATORS: translate only arg:, the rest are keywords */
-    N_("arg: false|true|ondemand|stls|requred|connection") },
+    N_("arg: false|true|ondemand|stls|required|connection") },
   { "tls", mu_cfg_section,
     NULL, mu_offsetof (struct imap4d_srv_config, tls_conf) },
   { NULL }
