@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2000, 2005, 2007-2008, 2010-2012, 2014-2018 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2020 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -318,7 +317,7 @@ typedef int (*mu_list_folder_t) (void *_item, void *_data,
 
        0                   - success,
        EINVAL              - _list or _fold is NULL,
-       MU_ERR_OUT_PTR_NULL - _return_code is NULL
+       MU_ERR_OUT_PTR_NULL - _return_value is NULL
        other value         - non-zero value returned by _fold.
 
      The _fold function is not allowed to alter the list it is being applied
