@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2010-2012, 2014-2017 Free Software Foundation, Inc.
+   Copyright (C) 2010-2020 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
@@ -19,6 +19,7 @@
 
 #include <mailutils/types.h>
 #include <mailutils/sys/stream.h>
+#include <mailutils/locus.h>
 
 struct _mu_log_stream
 {
@@ -30,7 +31,7 @@ struct _mu_log_stream
   int logmode;             /* Mode flags */
   int sevmask;             /* Mask out the output of severity level for
 			      these severities. */
-  struct mu_locus locus;   /* Location */
+  struct mu_locus_range locrange;  /* Location in the source file */
 };
 
 void _mu_log_stream_setup (struct _mu_log_stream *sp, mu_stream_t transport);
