@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2019 Free Software Foundation, Inc.
+   Copyright (C) 1999-2020 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -64,7 +64,7 @@ get_received_date (mu_message_t msg, struct tm *tm, struct mu_timezone *tz)
 		  ; years, are prohibited in SMTP and MUST NOT be used.
   */
   p = strchr (val, ';');
-  if (*p)
+  if (p)
     {
       p = mu_str_skip_class (p + 1, MU_CTYPE_SPACE);
       if (*p && mu_scan_datetime (p, MU_DATETIME_SCAN_RFC822, tm, tz, NULL) == 0)
