@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003-2004, 2007-2012, 2014-2015 Free Software
-   Foundation, Inc.
+   Copyright (C) 2003-2020 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -20,118 +19,33 @@
 # include <config.h>
 #endif
 
-#include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <string.h>
 
-#include <mailutils/error.h>
-#include <mailutils/mu_auth.h>
 #include <mailutils/tls.h>
 #include <mailutils/nls.h>
 #include <mailutils/stream.h>
+#include <mailutils/error.h>
 #include <mailutils/errno.h>
-#include <mailutils/util.h>
 #include <mailutils/property.h>
-
-struct mu_tls_module_config mu_tls_module_config = {
-#ifdef WITH_TLS
-  1 /* enable by default */
-#else
-  0
-#endif
-};
-  
-int
-mu_tls_module_init (enum mu_gocs_op op, void *data)
-{
-  switch (op)
-    {
-    case mu_gocs_op_set:
-      if (data)
-	memcpy (&mu_tls_module_config, data, sizeof mu_tls_module_config);
-      break;
-      
-    case mu_gocs_op_flush:
-      break;
-    }
-  return 0;
-}
-
-#ifdef WITH_TLS
+#include <mailutils/mu_auth.h>
 
 #include <gnutls/gnutls.h>
 #include <mailutils/sys/tls-stream.h>
 
-static gnutls_certificate_server_credentials x509_cred;
-
-/* Return: zero means NOT READY, one means READY */
-int
-mu_check_tls_environment (void)
-{
-  if (!mu_tls_module_config.enable)
-    return 0;
-  if (mu_tls_module_config.ssl_cert && mu_tls_module_config.ssl_key)
-    {
-      int rc = mu_file_safety_check (mu_tls_module_config.ssl_cert,
-				     mu_tls_module_config.ssl_cert_safety_checks,
-				     -1, NULL);
-      if (rc)
-	{
-	  mu_error ("%s: %s", mu_tls_module_config.ssl_cert, 
-		    mu_strerror (rc));
-	  return 0;
-	}
-      rc = mu_file_safety_check (mu_tls_module_config.ssl_key,
-				 mu_tls_module_config.ssl_key_safety_checks,
-				 -1, NULL);
-      if (rc)
-	{
-	  mu_error ("%s: %s", mu_tls_module_config.ssl_key, 
-		    mu_strerror (rc));
-	  return 0;
-	}
-
-      if (mu_tls_module_config.ssl_cafile)
-	{
-	  rc = mu_file_safety_check (mu_tls_module_config.ssl_cafile,
-			       mu_tls_module_config.ssl_cafile_safety_checks,
-				     -1, NULL);
-	  if (rc)
-	    {
-	      mu_error ("%s: %s", mu_tls_module_config.ssl_cafile, 
-			mu_strerror (rc));
-	      return 0;
-	    }
-	}
-    }
-  else
-    return 0;
-
-  return 1;
-}
-
-int mu_tls_enable = 0;
-
-#ifdef DEBUG_TLS
-void
+static void
 _mu_gtls_logger(int level, const char *text)
 {
-  mu_diag_output (MU_DIAG_DEBUG, "GnuTLS(%d): %s", level, text);
+  int len = strlen (text);
+  if (text[len-1] == '\n')
+    len--;
+  mu_diag_output (MU_DIAG_DEBUG, "GnuTLS(%d): %.*s", level, len, text);
 }
-#endif
 
 void
 mu_deinit_tls_libs (void)
 {
   if (mu_tls_enable)
-    {
-      if (x509_cred)
-	gnutls_certificate_free_credentials (x509_cred);
-      gnutls_global_deinit ();
-    }
+    gnutls_global_deinit ();
   mu_tls_enable = 0;
 }
 
@@ -142,270 +56,31 @@ _onexit_deinit (void *ptr MU_ARG_UNUSED)
 }
 
 int
-mu_init_tls_libs (int x509_setup)
+mu_init_tls_libs (void)
 {
   if (!mu_tls_enable)
     {
-      int rc;
-      if ((rc = gnutls_global_init ()) == GNUTLS_E_SUCCESS)
-	mu_tls_enable = 1;
+      int rc = gnutls_global_init ();
+      if (rc == GNUTLS_E_SUCCESS)
+	{
+	  mu_tls_enable = 1;
+	  mu_onexit (_onexit_deinit, NULL);
+	  if (mu_debug_level_p (MU_DEBCAT_TLS, MU_DEBUG_PROT))
+	    {
+	      gnutls_global_set_log_function (_mu_gtls_logger);
+	      gnutls_global_set_log_level (110);
+	    }
+	}
       else
 	{
 	  mu_error ("gnutls_global_init: %s", gnutls_strerror (rc));
-	  return 0;
 	}
-      mu_onexit (_onexit_deinit, NULL);
     }
-
-  if (x509_setup && !x509_cred)
-    {
-      gnutls_certificate_allocate_credentials (&x509_cred);
-      if (mu_tls_module_config.ssl_cafile)
-	gnutls_certificate_set_x509_trust_file (x509_cred,
-						mu_tls_module_config.ssl_cafile,
-						GNUTLS_X509_FMT_PEM);
-  
-      gnutls_certificate_set_x509_key_file (x509_cred,
-					    mu_tls_module_config.ssl_cert, 
-					    mu_tls_module_config.ssl_key,
-					    GNUTLS_X509_FMT_PEM);
-    }
-  
-#ifdef DEBUG_TLS
-  gnutls_global_set_log_function (_mu_gtls_logger);
-  gnutls_global_set_log_level (110);
-#endif
   return mu_tls_enable;
 }
-
-static char default_priority_string[] = "NORMAL";
-
-static gnutls_session_t
-initialize_tls_session (void)
-{
-  gnutls_session_t session = 0;
-
-  gnutls_init (&session, GNUTLS_SERVER);
-  gnutls_priority_set_direct (session,
-			      mu_tls_module_config.priorities
-			        ? mu_tls_module_config.priorities
-			        : default_priority_string,
-			      NULL);
-  gnutls_credentials_set (session, GNUTLS_CRD_CERTIFICATE, x509_cred);
-  gnutls_certificate_server_set_request (session, GNUTLS_CERT_REQUEST);
-
-  return session;
-}
-
 
-/* ************************* TLS Stream Support **************************** */
+/* TLS push & pull functions */
 
-static int
-_tls_io_close (mu_stream_t stream)
-{
-  struct _mu_tls_io_stream *sp = (struct _mu_tls_io_stream *) stream;
-  
-  return mu_stream_close (sp->transport);
-}
-
-static void
-_tls_io_done (struct _mu_stream *stream)
-{
-  struct _mu_tls_io_stream *sp = (struct _mu_tls_io_stream *) stream;
-  mu_stream_unref (sp->transport);
-}
-
-static int
-_tls_io_flush (struct _mu_stream *stream)
-{
-  struct _mu_tls_io_stream *sp = (struct _mu_tls_io_stream *) stream;
-  return mu_stream_flush (sp->transport);
-}
-
-static int
-_tls_io_read (struct _mu_stream *stream, char *buf, size_t bufsize,
-	      size_t *pnread)
-{
-  struct _mu_tls_io_stream *sp = (struct _mu_tls_io_stream *) stream;
-  int rc;
-  
-  if (sp->up->state != state_open)
-    return EINVAL;
-  rc = gnutls_record_recv (sp->up->session, buf, bufsize);
-  if (rc >= 0)
-    {
-      *pnread = rc;
-      return 0;
-    }
-  sp->up->tls_err = rc;
-  return EIO;
-}
-
-static int
-_tls_io_write (struct _mu_stream *stream, const char *buf, size_t bufsize,
-	    size_t *pnwrite)
-{
-  struct _mu_tls_io_stream *sp = (struct _mu_tls_io_stream *) stream;
-  int rc;
-  
-  if (sp->up->state != state_open)
-    return EINVAL;
-
-  /* gnutls_record_send() docs say:
-       If the EINTR is returned by the internal push function (write())
-       then GNUTLS_E_INTERRUPTED, will be returned. If GNUTLS_E_INTERRUPTED or
-       GNUTLS_E_AGAIN is returned you must call this function again, with the
-       same parameters. Otherwise the write operation will be
-       corrupted and the connection will be terminated. */
-    
-  do
-    rc = gnutls_record_send (sp->up->session, buf, bufsize);
-  while (rc == GNUTLS_E_INTERRUPTED || rc == GNUTLS_E_AGAIN);
-
-  if (rc < 0)
-    {
-      sp->up->tls_err = rc;
-      return EIO;
-    }
-
-  *pnwrite = rc;
-
-  return 0;
-}
-
-static int
-_tls_rd_wait (struct _mu_stream *stream, int *pflags, struct timeval *tvp)
-{
-  struct _mu_tls_io_stream *sp = (struct _mu_tls_io_stream *) stream;
-  int rc = EINVAL;
-  
-  if (*pflags == MU_STREAM_READY_RD)
-    rc = mu_stream_wait (sp->transport, pflags, tvp);
-  return rc;
-}
-
-static int
-_tls_wr_wait (struct _mu_stream *stream, int *pflags, struct timeval *tvp)
-{
-  struct _mu_tls_io_stream *sp = (struct _mu_tls_io_stream *) stream;
-  int rc = EINVAL;
-  
-  if (*pflags == MU_STREAM_READY_WR)
-    rc = mu_stream_wait (sp->transport, pflags, tvp);
-  return rc;
-}
-
-static int
-get_cipher_info (gnutls_session_t session, mu_property_t *pprop)
-{
-  mu_property_t prop;
-  const char *s;
-  int rc;
-
-  if (!pprop)
-    return EINVAL;
-
-  rc = mu_property_create_init (&prop, mu_assoc_property_init, NULL);
-  if (rc)
-    return rc;
-
-  s = gnutls_protocol_get_name (gnutls_protocol_get_version (session));
-  mu_property_set_value (prop, "protocol", s, 1);
-
-  s = gnutls_cipher_get_name (gnutls_cipher_get (session));
-  mu_property_set_value (prop, "cipher", s, 1);
-
-  s = gnutls_mac_get_name (gnutls_mac_get (session));
-  mu_property_set_value (prop, "mac", s, 1);
-
-  *pprop = prop;
-
-  return 0;
-}
-
-static int
-_tls_io_ioctl (struct _mu_stream *stream, int code, int opcode, void *arg)
-{
-  struct _mu_tls_io_stream *sp = (struct _mu_tls_io_stream *) stream;
-
-  switch (code)
-    {
-    case MU_IOCTL_TRANSPORT:
-      if (!arg)
-	return EINVAL;
-      else
-	{
-	  mu_transport_t *ptrans = arg;
-	  switch (opcode)
-	    {
-	    case MU_IOCTL_OP_GET:
-	      ptrans[0] = (mu_transport_t) sp->transport;
-	      ptrans[1] = NULL;
-	      break;
-
-	    case MU_IOCTL_OP_SET:
-	      return ENOSYS;
-
-	    default:
-	      return EINVAL;
-	    }
-	}
-      break;
-
-    case MU_IOCTL_TLSSTREAM:
-      switch (opcode)
-	{
-	case MU_IOCTL_TLS_GET_CIPHER_INFO:
-	  return get_cipher_info (sp->up->session, arg);
-
-	default:
-	  return EINVAL;
-	}
-      break;
-      
-    default:
-      return ENOSYS;
-    }
-  return 0;
-}
-
-static int
-_mu_tls_io_stream_create (mu_stream_t *pstream,
-			  mu_stream_t transport, int flags,
-			  struct _mu_tls_stream *master)
-{
-  struct _mu_tls_io_stream *sp;
-
-  sp = (struct _mu_tls_io_stream *)
-    _mu_stream_create (sizeof (*sp), (flags & MU_STREAM_RDWR) | _MU_STR_OPEN);
-  if (!sp)
-    return ENOMEM;
-
-  if (flags & MU_STREAM_READ)
-    {
-      sp->stream.read = _tls_io_read; 
-      sp->stream.wait = _tls_rd_wait;
-      mu_stream_set_buffer ((mu_stream_t) sp, mu_buffer_full, 0);
-    }
-  else
-    {
-      sp->stream.write = _tls_io_write;
-      sp->stream.wait = _tls_wr_wait;
-      mu_stream_set_buffer ((mu_stream_t) sp, mu_buffer_line, 0);
-    }
-  sp->stream.flush = _tls_io_flush;
-  sp->stream.close = _tls_io_close;
-  sp->stream.done = _tls_io_done; 
-  sp->stream.ctl = _tls_io_ioctl;
-
-  mu_stream_ref (transport);
-  sp->transport = transport;
-  sp->up = master;
-  *pstream = (mu_stream_t) sp;
-  return 0;
-}
-
-
 static ssize_t
 _tls_stream_pull (gnutls_transport_ptr_t fd, void *buf, size_t size)
 {
@@ -438,82 +113,125 @@ _tls_stream_push (gnutls_transport_ptr_t fd, const void *buf, size_t size)
   mu_stream_flush (stream);
   return size;
 }
-
 
+static char default_priority_string[] = "NORMAL";
+
 static int
-_tls_server_open (mu_stream_t stream)
+prep_session (mu_stream_t stream)
 {
   struct _mu_tls_stream *sp = (struct _mu_tls_stream *) stream;
-  int rc = 0;
+  gnutls_certificate_credentials_t cred = NULL;
   mu_transport_t transport[2];
-  
-  if (!mu_tls_module_config.enable)
-    return MU_ERR_DISABLED;
-  if (!stream || sp->state != state_init)
-    return EINVAL;
-
-  if (!mu_init_tls_libs (1))
-    return MU_ERR_FAILURE;
-  
-  sp->session = initialize_tls_session ();
-  mu_stream_ioctl (stream, MU_IOCTL_TRANSPORT, MU_IOCTL_OP_GET, transport);
-  gnutls_transport_set_ptr2 (sp->session,
-			     (gnutls_transport_ptr_t) transport[0],
-			     (gnutls_transport_ptr_t) transport[1]);
-  gnutls_transport_set_pull_function (sp->session, _tls_stream_pull);
-  gnutls_transport_set_push_function (sp->session, _tls_stream_push);
-  
-  rc = gnutls_handshake (sp->session);
-  if (rc < 0)
-    {
-      gnutls_deinit (sp->session);
-      sp->tls_err = rc;
-      return EIO;
-    }
-  sp->state = state_open;
-  return 0;
-}
-
-static int
-prepare_client_session (mu_stream_t stream)
-{
-  struct _mu_tls_stream *sp = (struct _mu_tls_stream *) stream;
   int rc;
-  mu_transport_t transport[2];
+  const char *errp;
 
-  gnutls_init (&sp->session, GNUTLS_CLIENT);
-  gnutls_priority_set_direct (sp->session,
-			      mu_tls_module_config.priorities
-			        ? mu_tls_module_config.priorities
-			        : default_priority_string,
-			      NULL);
-  gnutls_certificate_allocate_credentials (&x509_cred);
-  if (mu_tls_module_config.ssl_cafile)
+  if (!sp->cred)
     {
-      rc = gnutls_certificate_set_x509_trust_file (x509_cred,
-					      mu_tls_module_config.ssl_cafile,
-						   GNUTLS_X509_FMT_PEM);
-      if (rc < 0)
+      rc = gnutls_certificate_allocate_credentials (&cred);
+      if (rc)
 	{
+	  mu_debug (MU_DEBCAT_STREAM, MU_DEBUG_ERROR,
+		    ("gnutls_certificate_allocate_credentials: %s",
+		     gnutls_strerror (rc)));
 	  sp->tls_err = rc;
-	  return -1;
+	  return MU_ERR_FAILURE;
 	}
+  
+      if (sp->conf.ca_file)
+	{
+	  rc = gnutls_certificate_set_x509_trust_file (cred, sp->conf.ca_file,
+						       GNUTLS_X509_FMT_PEM);
+	  if (rc < 0)
+	    {
+	      mu_debug (MU_DEBCAT_STREAM, MU_DEBUG_ERROR,
+			("can't use X509 CA file %s: %s",
+			 sp->conf.ca_file,
+			 gnutls_strerror (rc)));
+	      goto cred_err;
+	    }
+	}
+  
+      if (sp->conf.cert_file && sp->conf.key_file)
+	{
+	  rc = gnutls_certificate_set_x509_key_file (cred,
+						     sp->conf.cert_file, 
+						     sp->conf.key_file,
+						     GNUTLS_X509_FMT_PEM);
+	  if (rc != GNUTLS_E_SUCCESS)
+	    {
+	      mu_debug (MU_DEBCAT_STREAM, MU_DEBUG_ERROR,
+			("can't use X509 cert/key pair (%s,%s): %s",
+			 sp->conf.cert_file,
+			 sp->conf.key_file,
+			 gnutls_strerror (rc)));
+	      goto cred_err;
+	    }
+	}
+      sp->cred = cred;
+    }
+  
+  rc = gnutls_init (&sp->session, sp->session_type);
+  if (rc != GNUTLS_E_SUCCESS)
+    {
+      mu_debug (MU_DEBCAT_STREAM, MU_DEBUG_ERROR,
+		("failed to initialize session: %s", gnutls_strerror (rc)));
+      goto cred_err;
     }
 
-  gnutls_credentials_set (sp->session, GNUTLS_CRD_CERTIFICATE, x509_cred);
-
-  mu_stream_ioctl (stream, MU_IOCTL_TRANSPORT, MU_IOCTL_OP_GET, transport);
+  rc = gnutls_priority_set_direct (sp->session,
+				   sp->conf.priorities
+				     ? sp->conf.priorities
+				     : default_priority_string,
+				   &errp);
+  if (rc != GNUTLS_E_SUCCESS)
+    {
+      mu_debug (MU_DEBCAT_STREAM, MU_DEBUG_ERROR,
+		("error setting priorities near %s: %s",
+		 errp, gnutls_strerror (rc)));
+      goto cred_err;
+    }
+  
+  rc = gnutls_credentials_set (sp->session, GNUTLS_CRD_CERTIFICATE, sp->cred);
+  if (rc)
+    {
+      mu_debug (MU_DEBCAT_STREAM, MU_DEBUG_ERROR,
+		("gnutls_credentials_set: %s", gnutls_strerror (rc)));
+      goto sess_err;
+    }
+  
+  if (sp->session_type == GNUTLS_SERVER)
+    gnutls_certificate_server_set_request (sp->session, GNUTLS_CERT_REQUEST);
+  
+  rc = mu_stream_ioctl (stream, MU_IOCTL_TRANSPORT, MU_IOCTL_OP_GET,
+			transport);
+  if (rc)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_stream_ioctl", NULL, rc);
+      abort (); /* should not happen */
+    }
+  
   gnutls_transport_set_ptr2 (sp->session,
 			     (gnutls_transport_ptr_t) transport[0],
 			     (gnutls_transport_ptr_t) transport[1]);
   gnutls_transport_set_pull_function (sp->session, _tls_stream_pull);
   gnutls_transport_set_push_function (sp->session, _tls_stream_push);
-      
+
   return 0;
+  
+ sess_err:
+  gnutls_deinit (sp->session);
+ cred_err:
+  if (sp->cred)
+    {
+      gnutls_certificate_free_credentials (sp->cred);
+      sp->cred = NULL;
+    }
+  sp->tls_err = rc;
+  return MU_ERR_FAILURE;
 }
 
 static int
-_tls_client_open (mu_stream_t stream)
+_tls_open (mu_stream_t stream)
 {
   struct _mu_tls_stream *sp = (struct _mu_tls_stream *) stream;
   int rc = 0;
@@ -522,32 +240,38 @@ _tls_client_open (mu_stream_t stream)
     {
     case state_closed:
       if (sp->session)
-	gnutls_deinit (sp->session);
-      /* FALLTHROUGH */
-      
-    case state_init:
-      if (!mu_init_tls_libs (0))
-	return MU_ERR_FAILURE;
-      prepare_client_session (stream);
-      rc = gnutls_handshake (sp->session);
-      if (rc < 0)
 	{
+	  gnutls_deinit (sp->session);
+	  sp->session = NULL;
+	}
+      /* FALLTHROUGH */
+
+    case state_init:
+      rc = prep_session (stream);
+      if (rc)
+	break;
+      rc = gnutls_handshake (sp->session);
+      if (rc != GNUTLS_E_SUCCESS)
+	{
+	  mu_debug (MU_DEBCAT_STREAM, MU_DEBUG_ERROR,
+		    ("gnutls_handshake: %s", gnutls_strerror (rc)));
 	  sp->tls_err = rc;
 	  gnutls_deinit (sp->session);
+	  sp->session = NULL;
 	  sp->state = state_init;
-	  return MU_ERR_FAILURE;
 	}
+      else
+	/* FIXME: if (ssl_cafile) verify_certificate (s->session); */
+	sp->state = state_open;
       break;
 
     default:
-      return MU_ERR_FAILURE;
+      rc = MU_ERR_BADOP;
     }
-
-  /* FIXME: if (ssl_cafile) verify_certificate (s->session); */
-  sp->state = state_open;
-  return 0;
+  return rc;
 }
-
+
+/* TLS stream */
 static int
 _tls_read (struct _mu_stream *str, char *buf, size_t bufsize,
 	   size_t *pnread)
@@ -564,6 +288,34 @@ _tls_write (struct _mu_stream *str, const char *buf, size_t bufsize,
   return mu_stream_write (sp->transport[1], buf, bufsize, pnwrite);
 }
 
+static int
+get_cipher_info (gnutls_session_t session, mu_property_t *pprop)
+{
+  mu_property_t prop;
+  const char *s;
+  int rc;
+
+  if (!pprop)
+    return EINVAL;
+
+  rc = mu_property_create_init (&prop, mu_assoc_property_init, NULL);
+  if (rc)
+    return rc;
+
+  s = gnutls_protocol_get_name (gnutls_protocol_get_version (session));
+  mu_property_set_value (prop, "protocol", s, 1);
+
+  s = gnutls_cipher_get_name (gnutls_cipher_get (session));
+  mu_property_set_value (prop, "cipher", s, 1);
+
+  s = gnutls_mac_get_name (gnutls_mac_get (session));
+  mu_property_set_value (prop, "mac", s, 1);
+
+  *pprop = prop;
+
+  return 0;
+}
+
 static int
 _tls_ioctl (struct _mu_stream *stream, int code, int opcode, void *arg)
 {
@@ -637,6 +389,9 @@ _tls_ioctl (struct _mu_stream *stream, int code, int opcode, void *arg)
 	}
       break;
       
+    case MU_IOCTL_TCPSTREAM:
+      return mu_stream_ioctl (sp->transport[0], code, opcode, arg);
+			      
     default:
       return ENOSYS;
     }
@@ -679,17 +434,18 @@ _tls_close (mu_stream_t stream)
   return 0;
 }
 
+static void free_conf (struct mu_tls_config *conf);
+
 static void
 _tls_done (struct _mu_stream *stream)
 {
   struct _mu_tls_stream *sp = (struct _mu_tls_stream *) stream;
   
-  if (sp->session && sp->state == state_closed)
-    {
-      gnutls_deinit (sp->session);
-      sp->state = state_destroyed;
-    }
-
+  if (sp->session)
+    gnutls_deinit (sp->session);
+  if (sp->cred)
+    gnutls_certificate_free_credentials (sp->cred);
+  free_conf (&sp->conf);
   mu_stream_destroy (&sp->transport[0]);
   mu_stream_destroy (&sp->transport[1]);
 }
@@ -705,24 +461,121 @@ _tls_error_string (struct _mu_stream *stream, int rc)
   return mu_strerror (rc);
 }
 
+static void
+free_conf (struct mu_tls_config *conf)
+{
+  free (conf->cert_file);
+  free (conf->key_file);
+  free (conf->ca_file);
+}
+
 static int
-_mu_tls_stream_create (mu_stream_t *pstream,
-		       int (*openfn) (mu_stream_t stream),
-		       mu_stream_t strin, mu_stream_t strout, int flags)
+copy_conf (struct mu_tls_config *dst, struct mu_tls_config const *src)
+{
+  if (src->cert_file)
+    {
+      dst->cert_file = strdup (src->cert_file);
+      if (!dst->cert_file)
+	return errno;
+    }
+  else
+    dst->cert_file = NULL;
+
+  if (src->key_file)
+    {
+      dst->key_file = strdup (src->key_file);
+      if (!dst->cert_file)
+	{
+	  int rc = errno;
+	  free (dst->cert_file);
+	  return rc;
+	}
+    }
+  else
+    dst->key_file = NULL;
+  
+  if (src->ca_file)
+    {
+      dst->ca_file = strdup (src->ca_file);
+      if (!dst->ca_file)
+	{
+	  int rc = errno;
+	  free (dst->cert_file);
+	  free (dst->key_file);
+	  return rc;
+	}
+    }
+  else
+    dst->ca_file = NULL;
+
+  return 0;
+}
+
+int
+mu_tls_stream_create (mu_stream_t *pstream,
+		      mu_stream_t strin, mu_stream_t strout,
+		      struct mu_tls_config const *conf,
+		      enum mu_tls_type type,
+		      int flags)
 {
   struct _mu_tls_stream *sp;
   int rc;
   mu_stream_t stream;
+  int session_type;
+  
+  if (!pstream)
+    return MU_ERR_OUT_PTR_NULL;
+  if (!conf || !strin || !strout)
+    return EINVAL;
+
+  if (!mu_init_tls_libs ())
+    return ENOSYS;
+
+  switch (mu_tls_config_check (conf, 1))
+    {
+    case MU_TLS_CONFIG_OK:
+    case MU_TLS_CONFIG_NULL:
+      break;
+    case MU_TLS_CONFIG_UNSAFE:
+      return EACCES;
+    case MU_TLS_CONFIG_FAIL:
+      return ENOENT;
+    }
+  
+  switch (type)
+    {
+    case MU_TLS_CLIENT:
+      session_type = GNUTLS_CLIENT;
+      break;
+      
+    case MU_TLS_SERVER:
+      session_type = GNUTLS_SERVER;
+      break;
+      
+    default:
+      return EINVAL;
+    }
   
   sp = (struct _mu_tls_stream *)
     _mu_stream_create (sizeof (*sp), MU_STREAM_RDWR);
   if (!sp)
     return ENOMEM;
+  sp->session_type = session_type;
+  sp->state = state_init;
+  sp->session = NULL;
+  sp->cred = NULL;
 
+  rc = copy_conf (&sp->conf, conf);
+  if (rc)
+    {
+      free (sp);
+      return rc;
+    }
+	  
   sp->stream.read = _tls_read; 
   sp->stream.write = _tls_write;
   sp->stream.flush = _tls_flush;
-  sp->stream.open = openfn; 
+  sp->stream.open = _tls_open; 
   sp->stream.close = _tls_close;
   sp->stream.done = _tls_done; 
   sp->stream.ctl = _tls_ioctl;
@@ -731,52 +584,43 @@ _mu_tls_stream_create (mu_stream_t *pstream,
 
   mu_stream_set_buffer (strin, mu_buffer_none, 0);
   mu_stream_set_buffer (strout, mu_buffer_none, 0);
-  rc = _mu_tls_io_stream_create (&sp->transport[0], strin, MU_STREAM_READ, sp);
-  if (rc)
-    {
-      free (sp);
-      return rc;
-    }
-     
-  rc = _mu_tls_io_stream_create (&sp->transport[1], strout, MU_STREAM_WRITE,
-                                 sp);
-  if (rc)
-    {
-      free (sp);
-      free (sp->transport[0]);
-      return rc;
-    }
 
   stream = (mu_stream_t) sp;
+  
+  rc = mu_tls_io_stream_create (&sp->transport[0], strin,
+				MU_STREAM_READ, sp);
+  if (rc)
+    goto err;
+  
+  rc = mu_tls_io_stream_create (&sp->transport[1], strout,
+				MU_STREAM_WRITE, sp);
+  if (rc)
+    goto err;
+  
   mu_stream_set_buffer (stream, mu_buffer_line, 0);
   rc = mu_stream_open (stream);
   if (rc)
-    mu_stream_destroy (&stream);
-  else
-    *pstream = stream;
+    goto err;
+  *pstream = stream;
+  return 0;
+  
+ err:
+  mu_stream_destroy (&stream);
   return rc;
 }
-
-int
-mu_tls_server_stream_create (mu_stream_t *pstream,
-			     mu_stream_t strin, mu_stream_t strout, int flags)
-{
-  return _mu_tls_stream_create (pstream,
-				_tls_server_open,
-				strin, strout, flags);
-}
-
+
 int
 mu_tls_client_stream_create (mu_stream_t *pstream,
 			     mu_stream_t strin, mu_stream_t strout, int flags)
 {
-  return _mu_tls_stream_create (pstream,
-				_tls_client_open,
-				strin, strout, flags);
+  struct mu_tls_config conf = {
+    .cert_file = NULL,
+    .key_file = NULL,
+    .ca_file = NULL,
+    .priorities = NULL
+  };
+
+  return mu_tls_stream_create (pstream, strin, strout, &conf, MU_TLS_CLIENT,
+			       flags);
 }
 
-
-#endif /* WITH_TLS */
-
-/* EOF */
-
