@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2019 Free Software Foundation, Inc.
+   Copyright (C) 1999-2020 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -371,7 +371,6 @@ compl_concat (char **a, char **b)
   return ret;
 }
 
-static char *msgtype_generator (const char *text, int state);
 static char *header_generator (const char *text, int state);
 
 /* Internal completion generator for commands that take a message list
@@ -685,34 +684,6 @@ folder_generator (const char *text, int state)
   return filegen_next (&fg);
 }
 
-static char *
-msgtype_generator (const char *text, int state)
-{
-  /* Allowed message types, plus '/'. The latter can folow a colon,
-     meaning body lookup */
-  static char types[] = "dnorTtu/";
-  static int i;
-  char c;
-
-  if (!state)
-    {
-      i = 0;
-    }
-  while ((c = types[i]))
-    {
-      i++;
-      if (!text[1] || text[1] == c)
-	{
-	  char *s = mu_alloc (3);
-	  s[0] = ':';
-	  s[1] = c;
-	  s[2] = 0;
-	  return s;
-	}
-    }
-  return NULL;
-}
-
 static char *
 header_generator (const char *text, int state)
 {
