@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2002, 2005, 2007, 2009-2012, 2014-2018 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2020 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -36,7 +35,7 @@ mail_print_msg (msgset_t *mspec, mu_message_t mesg, void *data)
   int status;
   
   mu_message_lines (mesg, &lines);
-  if (mailvar_is_true ("showenvelope"))
+  if (mailvar_is_true (mailvar_name_showenvelope))
     lines++;
   
   /* If it is POP or IMAP the lines number is not known, so try
@@ -57,8 +56,8 @@ mail_print_msg (msgset_t *mspec, mu_message_t mesg, void *data)
 
   out = open_pager (lines);
 
-  if (mailvar_is_true ("showenvelope"))
-    print_envelope (mspec, mesg, "From");
+  if (mailvar_is_true (mailvar_name_showenvelope))
+    print_stream_envelope (out, mspec, mesg, "From");
   
   if (*(int *) data) /* print was called with a lowercase 'p' */
     {
