@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2001, 2005, 2007, 2010-2012, 2014-2017 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2020 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,7 +28,7 @@ top0 (msgset_t *mspec, mu_message_t msg, void *data)
   size_t size = 0, n;
   int lines;
   
-  if (mailvar_get (&lines, "toplines", mailvar_type_number, 1)
+  if (mailvar_get (&lines, mailvar_name_toplines, mailvar_type_number, 1)
       || lines < 0)
     return 1;
 
