@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2003, 2005-2007, 2009-2012, 2014-2017 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2020 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -726,6 +725,7 @@ mh_format_str (mh_format_t fmt, char *str, size_t width, char **pstr)
   buf = mu_alloc (size + 1);
   MU_ASSERT (mu_stream_seek (outstr, 0, MU_SEEK_SET, NULL));
   MU_ASSERT (mu_stream_read (outstr, buf, size, NULL));
+  buf[size] = 0;
 
   *pstr = buf;
   
